(******************************************************************************)
(*   __               ___     CAVE: Concurrent Algorithm VErifier             *)
(*  /     /\  \    / |        Copyright (c) Viktor Vafeiadis                  *)
(* |     /--\  \  /  |---                                                     *)
(*  \__ /    \  \/   |___     See LICENSE.txt for license.                    *)
(*                                                                            *)
(******************************************************************************)
(** Rely-guarantee actions *)
open Exp
open Assertions
open Genarith

(** action name, context pure, context spatial, action pre, action post *)
type myaction = (string * Pure.t * can_spatial * can_spatial * can_spatial)

type stab_env = {
    st_prover : prover ;
    st_abstraction : abstraction ;
    st_lin_hook : uform list -> uform list ;
    st_lin_exit : uform list -> unit (* raise exception to exit *) 
  }

type guar_env = {
    mutable ge_actions : (Misc.component (* resource name *) * myaction) list ;
    mutable ge_changed : bool 
  }

val mk_guar_env : unit -> guar_env

(* -------------------------------------------------------------------------- *)
(** {2 Configuration parameters & statistics} *)
(* -------------------------------------------------------------------------- *)

val verbose : int ref 

val remember_result_hack : bool ref 
val lock_abstraction_heuristic : bool ref 

val args : (Arg.key * Arg.spec * Arg.doc) list

(** total number of iterations for all the calls to mk_stable_single *)
val stabilisation_iterations : int ref

val pp_actions : (Misc.component * myaction) list -> unit

(* -------------------------------------------------------------------------- *)
(** {2 Stabilisation} *)
(* -------------------------------------------------------------------------- *)

(** [make_stable env cp rely] returns a [cp'] such that [cp |- cp'] and [stable_under cp rely] *)
val make_stable : stab_env -> cprop -> act list -> cprop

(** Compute [eq] such that [ep |- eq] and [stable_under ep rely] *)
val stabilize : stab_env -> (Misc.component * act list) -> eprop -> eprop

(* -------------------------------------------------------------------------- *)
(** {2 Action registration} *)
(* -------------------------------------------------------------------------- *)

(** Add action [cp_ctx | cp ~~> cq] to the guarantee. *)
val register_guarantee : 
  guar_env -> prover -> abstraction -> (*glob*)IdSet.t -> (*params*)IdSet.t ->
  (string * Assertions.cprop) list ref -> 
  Misc.component -> cprop -> cprop -> cprop -> unit

val update_res_ht : guar_env -> (Misc.component, (act list * act list)) Hashtbl.t -> unit

